import pandas as pd
import numpy as np

np.random.seed(1)

N = 120  # un poco más de 100

df = pd.DataFrame({
    "fecha": pd.date_range("2024-01-01", periods=N, freq="D"),
    "contacto_efectivo/contacto": np.random.uniform(0.45, 0.75, N),
    "retenido/contacto_efectivo": np.random.uniform(0.20, 0.45, N),
    "%nivel_de_servicio": np.random.uniform(0.70, 0.95, N),
    "tmo_contacto_efectivo_en_segundos": np.random.normal(2000, 300, N),
    "ftes": np.random.randint(25, 50, N)
})

df.to_csv("demo_data.csv", index=False)
df.head()



